@@ -115,6 +115,12 @@ gedit_gio_document_saver_dispose (GObject *object)
 		priv->stream = NULL;
 	}
 
+	if (priv->input != NULL)
+	{
+		g_object_unref (priv->input);
+		priv->input = NULL;
+	}
+
 	G_OBJECT_CLASS (gedit_gio_document_saver_parent_class)->dispose (object);
 }
 
@@ -146,7 +152,7 @@ gedit_gio_document_saver_class_init (GeditGioDocumentSaverClass *klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	GeditDocumentSaverClass *saver_class = GEDIT_DOCUMENT_SAVER_CLASS (klass);
 
-	object_class->finalize = gedit_gio_document_saver_dispose;
+	object_class->dispose = gedit_gio_document_saver_dispose;
 
 	saver_class->save = gedit_gio_document_saver_save;
 	saver_class->get_file_size = gedit_gio_document_saver_get_file_size;
@@ -168,12 +174,12 @@ static void
 remote_save_completed_or_failed (GeditGioDocumentSaver *gvsaver, 
 				 AsyncData 	       *async)
 {
-	if (async)
-		async_data_free (async);
-
 	gedit_document_saver_saving (GEDIT_DOCUMENT_SAVER (gvsaver),
 				     TRUE,
 				     gvsaver->priv->error);
+
+	if (async)
+		async_data_free (async);
 }
 
 static void
@@ -186,15 +192,15 @@ async_failed (AsyncData *async,
 }
 
 static void
-remote_reget_info_cb (GFile        *source,
-		      GAsyncResult *res,
-		      AsyncData    *async)
+remote_get_info_cb (GFile        *source,
+		    GAsyncResult *res,
+		    AsyncData    *async)
 {
 	GeditGioDocumentSaver *saver;
 	GFileInfo *info;
 	GError *error = NULL;
 
-	saver = async->saver;
+	gedit_debug (DEBUG_SAVER);
 
 	/* check cancelled state manually */
 	if (g_cancellable_is_cancelled (async->cancellable))
@@ -203,6 +209,8 @@ remote_reget_info_cb (GFile        *source,
 		return;
 	}
 	
+	saver = async->saver;
+
 	gedit_debug_message (DEBUG_SAVER, "Finished query info on file");
 	info = g_file_query_info_finish (source, res, &error);
 
@@ -228,7 +236,9 @@ close_async_ready_get_info_cb (GOutputStream *stream,
 			       AsyncData     *async)
 {
 	GError *error = NULL;
-	
+
+	gedit_debug (DEBUG_SAVER);
+
 	/* check cancelled state manually */
 	if (g_cancellable_is_cancelled (async->cancellable))
 	{
@@ -245,98 +255,44 @@ close_async_ready_get_info_cb (GOutputStream *stream,
 		async_failed (async, error);
 		return;
 	}
-	
+
+	/* get the file info: note we cannot use 
+	 * g_file_output_stream_query_info_async since it is not able to get the
+	 * content type etc, beside it is not supported by gvfs.
+	 * I'm not sure this is actually necessary, can't we just use
+	 * g_content_type_guess (since we have the file name and the data)
+	 */
 	gedit_debug_message (DEBUG_SAVER, "Query info on file");
 	g_file_query_info_async (async->saver->priv->gfile,
 			         REMOTE_QUERY_ATTRIBUTES,
 			         G_FILE_QUERY_INFO_NONE,
 			         G_PRIORITY_HIGH,
 			         async->cancellable,
-			         (GAsyncReadyCallback) remote_reget_info_cb,
+			         (GAsyncReadyCallback) remote_get_info_cb,
 			         async);
 }
 
 static void
-close_async_ready_cb (GOutputStream *stream,
-		      GAsyncResult  *res,
-		      AsyncData     *async)
+write_complete (AsyncData *async)
 {
 	GError *error = NULL;
 
-	/* check cancelled state manually */
-	if (g_cancellable_is_cancelled (async->cancellable))
+	/* first we close the input stream */
+	gedit_debug_message (DEBUG_SAVER, "Close input stream");
+	if (!g_input_stream_close (async->saver->priv->input,
+				   async->cancellable, &error))
 	{
-		async_data_free (async);
+		gedit_debug_message (DEBUG_SAVER, "Closing input stream error: %s", error->message);
+		async_failed (async, error);
 		return;
 	}
 
-	if (!g_output_stream_close_finish (stream, res, &error))
-	{
-		g_propagate_error (&async->saver->priv->error, error);
-	}
-
-	remote_save_completed_or_failed (async->saver, async);
-}
-
-static void
-remote_get_info_cb (GFileOutputStream *stream,
-		    GAsyncResult      *res,
-		    AsyncData         *async)
-{
-	GeditGioDocumentSaver *saver;
-	GFileInfo *info;
-	GError *error = NULL;
-	GAsyncReadyCallback next_callback;
-
-	saver = async->saver;
-
-	/* check cancelled state manually */
-	if (g_cancellable_is_cancelled (async->cancellable))
-	{
-		async_data_free (async);
-		return;
-	}
-	
-	gedit_debug_message (DEBUG_SAVER, "Finishing info query");
-	info = g_file_output_stream_query_info_finish (stream, res, &error);
-	gedit_debug_message (DEBUG_SAVER, "Query info result: %s", info ? "ok" : "fail");
-
-	if (info == NULL)
-	{
-		if (error->code == G_IO_ERROR_NOT_SUPPORTED || error->code == G_IO_ERROR_CLOSED)
-		{
-			gedit_debug_message (DEBUG_SAVER, "Query info not supported on stream, trying on file");
-
-			/* apparently the output stream does not support query info.
-			 * we're forced to restat on the file. But first we make sure
-			 * to close the stream
-			 */
-			g_error_free (error);
-			next_callback = (GAsyncReadyCallback) close_async_ready_get_info_cb;
-		}
-		else
-		{
-			gedit_debug_message (DEBUG_SAVER, "Query info failed: %s", error->message);
-			g_propagate_error (&saver->priv->error, error);
-
-			next_callback = (GAsyncReadyCallback) close_async_ready_cb;
-		}
-	}
-	else
-	{
-		if (GEDIT_DOCUMENT_SAVER (saver)->info != NULL)
-			g_object_unref (GEDIT_DOCUMENT_SAVER (saver)->info);
-
-		GEDIT_DOCUMENT_SAVER (saver)->info = info;
-
-		next_callback = (GAsyncReadyCallback) close_async_ready_cb;
-	}
-
-	/* Close the main stream so the file stream is also closed */
-	g_output_stream_close_async (G_OUTPUT_STREAM (saver->priv->stream),
+	/* now we close the output stream */
+	gedit_debug_message (DEBUG_SAVER, "Close output stream");
+	g_output_stream_close_async (async->saver->priv->stream,
 				     G_PRIORITY_HIGH,
 				     async->cancellable,
-				     next_callback,
+				     (GAsyncReadyCallback)close_async_ready_get_info_cb,
 				     async);
 }
 
@@ -344,31 +300,6 @@ remote_get_info_cb (GFileOutputStream *stream,
 static void read_file_chunk (AsyncData *async);
 static void write_file_chunk (AsyncData *async);
 
-static void
-write_complete (AsyncData *async)
-{
-	GOutputStream *file_stream;
-
-	/* document is succesfully saved. we know requery for the mime type and
-	 * the mtime. I'm not sure this is actually necessary, can't we just use
-	 * g_content_type_guess (since we have the file name and the data)
-	 */
-	gedit_debug_message (DEBUG_SAVER, "Write complete, query info on stream");
-
-	if (G_IS_FILE_OUTPUT_STREAM (async->saver->priv->stream))
-		file_stream = async->saver->priv->stream;
-	else
-		file_stream = g_filter_output_stream_get_base_stream (G_FILTER_OUTPUT_STREAM (async->saver->priv->stream));
-	
-	/* query info on the stream */
-	g_file_output_stream_query_info_async (G_FILE_OUTPUT_STREAM (file_stream),
-					       REMOTE_QUERY_ATTRIBUTES,
-					       G_PRIORITY_HIGH,
-					       async->cancellable,
-					       (GAsyncReadyCallback) remote_get_info_cb,
-					       async);
-}
-
 static void
 async_write_cb (GOutputStream *stream,
 		GAsyncResult  *res,
@@ -378,6 +309,8 @@ async_write_cb (GOutputStream *stream,
 	gssize bytes_written;
 	GError *error = NULL;
 
+	gedit_debug (DEBUG_SAVER);
+
 	/* Check cancelled state manually */
 	if (g_cancellable_is_cancelled (async->cancellable))
 	{
@@ -403,6 +336,7 @@ async_write_cb (GOutputStream *stream,
 	if (async->written != async->read)
 	{
 		write_file_chunk (async);
+		return;
 	}
 
 	/* note that this signal blocks the write... check if it isn't
@@ -432,23 +366,22 @@ write_file_chunk (AsyncData *async)
 }
 
 static void
-async_read_cb (GInputStream *stream,
-	       GAsyncResult *res,
-	       AsyncData    *async)
+read_file_chunk (AsyncData *async)
 {
 	GeditGioDocumentSaver *gvsaver;
 	GeditDocumentInputStream *dstream;
 	GError *error = NULL;
 
-	if (g_cancellable_is_cancelled (async->cancellable))
-	{
-		async_data_free (async);
-		return;
-	}
-
 	gvsaver = async->saver;
+	async->written = 0;
 
-	async->read = g_input_stream_read_finish (stream, res, &error);
+	/* we use sync methods on doc stream since it is in memory. Using async
+	   would be racy and we can endup with invalidated iters */
+	async->read = g_input_stream_read (gvsaver->priv->input,
+					   async->buffer,
+					   WRITE_CHUNK_SIZE,
+					   async->cancellable,
+					   &error);
 
 	if (error != NULL)
 	{
@@ -464,29 +397,12 @@ async_read_cb (GInputStream *stream,
 	}
 
 	/* Get how many chars have been read */
-	dstream = GEDIT_DOCUMENT_INPUT_STREAM (stream);
-	gvsaver->priv->bytes_written += gedit_document_input_stream_tell (dstream);
+	dstream = GEDIT_DOCUMENT_INPUT_STREAM (gvsaver->priv->input);
+	gvsaver->priv->bytes_written = gedit_document_input_stream_tell (dstream);
 
 	write_file_chunk (async);
 }
 
-static void
-read_file_chunk (AsyncData *async)
-{
-	GeditGioDocumentSaver *gvsaver;
-
-	gvsaver = async->saver;
-	async->written = 0;
-
-	g_input_stream_read_async (gvsaver->priv->input,
-				   async->buffer,
-				   WRITE_CHUNK_SIZE,
-				   G_PRIORITY_HIGH,
-				   async->cancellable,
-				   (GAsyncReadyCallback) async_read_cb,
-				   async);
-}
-
 static void
 async_replace_ready_callback (GFile        *source,
 			      GAsyncResult *res,
@@ -498,6 +414,8 @@ async_replace_ready_callback (GFile        *source,
 	GFileOutputStream *file_stream;
 	GError *error = NULL;
 
+	gedit_debug (DEBUG_SAVER);
+
 	/* Check cancelled state manually */
 	if (g_cancellable_is_cancelled (async->cancellable))
 	{
@@ -549,6 +467,8 @@ static void
 begin_write (AsyncData *async)
 {
 	GeditGioDocumentSaver *gvsaver;
+	GeditDocumentSaver *saver;
+	gboolean backup;
 
 	gedit_debug_message (DEBUG_SAVER, "Start replacing file contents");
 
@@ -556,14 +476,18 @@ begin_write (AsyncData *async)
 	 * backup as of yet
 	 */
 	gvsaver = async->saver;
+	saver = GEDIT_DOCUMENT_SAVER (gvsaver);
+
+	/* Do not make backups for remote files so they do not clutter remote systems */
+	backup = (saver->keep_backup && gedit_document_is_local (saver->document));
 
 	gedit_debug_message (DEBUG_SAVER, "File contents size: %" G_GINT64_FORMAT, gvsaver->priv->size);
 	gedit_debug_message (DEBUG_SAVER, "Calling replace_async");
+	gedit_debug_message (DEBUG_SAVER, backup ? "Keep backup" : "Discard backup");
 
-	/* FIXME: when do we want to make a backup? */
-	g_file_replace_async (gvsaver->priv->gfile, 
+	g_file_replace_async (gvsaver->priv->gfile,
 			      NULL,
-			      FALSE,
+			      backup,
 			      G_FILE_CREATE_NONE,
 			      G_PRIORITY_HIGH,
 			      async->cancellable,
@@ -578,7 +502,9 @@ mount_ready_callback (GFile        *file,
 {
 	GError *error = NULL;
 	gboolean mounted;
-	
+
+	gedit_debug (DEBUG_SAVER);
+
 	/* manual check for cancelled state */
 	if (g_cancellable_is_cancelled (async->cancellable))
 	{
@@ -629,7 +555,9 @@ check_modification_callback (GFile        *source,
 	GeditGioDocumentSaver *gvsaver;
 	GError *error = NULL;
 	GFileInfo *info;
-	
+
+	gedit_debug (DEBUG_SAVER);
+
 	/* manually check cancelled state */
 	if (g_cancellable_is_cancelled (async->cancellable))
 	{
