@@ -50,7 +50,6 @@
 #include "gedit-document.h"
 #include "gedit-prefs-manager.h"
 #include "gedit-debug.h"
-#include "gedit-convert.h"
 
 /* For the workspace/viewport stuff */
 #ifdef GDK_WINDOWING_X11
@@ -107,6 +106,16 @@ gedit_utils_uri_has_writable_scheme (const gchar *uri)
 	return res;
 }
 
+static void
+widget_get_origin (GtkWidget *widget, gint *x, gint *y)
+
+{
+	GdkWindow *window;
+
+	window = gtk_widget_get_window (widget);
+	gdk_window_get_origin (window, x, y);
+}
+
 void
 gedit_utils_menu_position_under_widget (GtkMenu  *menu,
 					gint     *x,
@@ -114,22 +123,24 @@ gedit_utils_menu_position_under_widget (GtkMenu  *menu,
 					gboolean *push_in,
 					gpointer  user_data)
 {
-	GtkWidget *w = GTK_WIDGET (user_data);
+	GtkWidget *widget;
 	GtkRequisition requisition;
 
-	gdk_window_get_origin (w->window, x, y);
+	widget = GTK_WIDGET (user_data);
+	widget_get_origin (widget, x, y);
+
 	gtk_widget_size_request (GTK_WIDGET (menu), &requisition);
 
-	if (gtk_widget_get_direction (w) == GTK_TEXT_DIR_RTL)
+	if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL)
 	{
-		*x += w->allocation.x + w->allocation.width - requisition.width;
+		*x += widget->allocation.x + widget->allocation.width - requisition.width;
 	}
 	else
 	{
-		*x += w->allocation.x;
+		*x += widget->allocation.x;
 	}
 
-	*y += w->allocation.y + w->allocation.height;
+	*y += widget->allocation.y + widget->allocation.height;
 
 	*push_in = TRUE;
 }
@@ -157,7 +168,7 @@ gedit_utils_menu_position_under_tree_view (GtkMenu  *menu,
 		GtkTreePath *path;
 		GdkRectangle rect;
 
-		gdk_window_get_origin (GTK_WIDGET (tree)->window, x, y);
+		widget_get_origin (GTK_WIDGET (tree), x, y);
 			
 		path = gtk_tree_model_get_path (model, &iter);
 		gtk_tree_view_get_cell_area (tree, path,
@@ -493,7 +504,7 @@ gedit_warning (GtkWindow *parent, const gchar *format, ...)
 	g_return_if_fail (format != NULL);
 
 	if (parent != NULL)
-		wg = parent->group;
+		wg = gtk_window_get_group (parent);
 		
 	va_start (args, format);
 	str = g_strdup_vprintf (format, args);
@@ -567,9 +578,10 @@ gedit_utils_escape_underscores (const gchar* text,
 
 /* the following functions are taken from eel */
 
-gchar *
-gedit_utils_str_middle_truncate (const gchar *string,
-				 guint        truncate_length)
+static gchar *
+gedit_utils_str_truncate (const gchar *string,
+			  guint        truncate_length,
+			  gboolean middle)
 {
 	GString     *truncated;
 	guint        length;
@@ -602,17 +614,41 @@ gedit_utils_str_middle_truncate (const gchar *string,
 	}
 
 	/* Find the 'middle' where the truncation will occur. */
-	num_left_chars = (truncate_length - delimiter_length) / 2;
-	right_offset = n_chars - truncate_length + num_left_chars + delimiter_length;
+	if (middle)
+	{
+		num_left_chars = (truncate_length - delimiter_length) / 2;
+		right_offset = n_chars - truncate_length + num_left_chars + delimiter_length;
 
-	truncated = g_string_new_len (string,
-				      g_utf8_offset_to_pointer (string, num_left_chars) - string);
-	g_string_append (truncated, delimiter);
-	g_string_append (truncated, g_utf8_offset_to_pointer (string, right_offset));
-		
+		truncated = g_string_new_len (string,
+					      g_utf8_offset_to_pointer (string, num_left_chars) - string);
+		g_string_append (truncated, delimiter);
+		g_string_append (truncated, g_utf8_offset_to_pointer (string, right_offset));
+	}
+	else
+	{
+		num_left_chars = truncate_length - delimiter_length;
+		truncated = g_string_new_len (string,
+					      g_utf8_offset_to_pointer (string, num_left_chars) - string);
+		g_string_append (truncated, delimiter);
+	}
+	
 	return g_string_free (truncated, FALSE);
 }
 
+gchar *
+gedit_utils_str_middle_truncate (const gchar *string,
+				 guint        truncate_length)
+{
+	return gedit_utils_str_truncate (string, truncate_length, TRUE);
+}
+
+gchar *
+gedit_utils_str_end_truncate (const gchar *string,
+			      guint        truncate_length)
+{
+	return gedit_utils_str_truncate (string, truncate_length, FALSE);
+}
+
 gchar *
 gedit_utils_make_valid_utf8 (const char *name)
 {
@@ -665,7 +701,7 @@ gedit_utils_uri_get_dirname (const gchar *uri)
 
 	/* CHECK: does it work with uri chaining? - Paolo */
 	str = g_path_get_dirname (uri);
-	g_return_val_if_fail (str != NULL, ".");
+	g_return_val_if_fail (str != NULL, g_strdup ("."));
 
 	if ((strlen (str) == 1) && (*str == '.'))
 	{
@@ -866,7 +902,7 @@ gedit_utils_get_window_workspace (GtkWindow *gtkwindow)
 	g_return_val_if_fail (GTK_IS_WINDOW (gtkwindow), 0);
 	g_return_val_if_fail (GTK_WIDGET_REALIZED (GTK_WIDGET (gtkwindow)), 0);
 
-	window = GTK_WIDGET (gtkwindow)->window;
+	window = gtk_widget_get_window (GTK_WIDGET (gtkwindow));
 	display = gdk_drawable_get_display (window);
 
 	gdk_error_trap_push ();
@@ -1086,22 +1122,27 @@ gedit_utils_get_ui_objects (const gchar  *filename,
 	builder = gtk_builder_new ();
 	
 	if (root_objects != NULL)
+	{
 		gtk_builder_add_objects_from_file (builder, 
 						   filename, 
 						   root_objects, 
 						   &error);
+	}
 	else
+	{
 		gtk_builder_add_from_file (builder,
 					   filename,
 					   &error);
+	}
 
 	if (error != NULL)
 	{
-		*error_widget = handle_builder_error (_("Unable to open ui file %s. Error: %s"),
+		*error_widget = handle_builder_error (_("Unable to open UI file %s. Error: %s"),
 						      filename_markup,
 						      error->message);
 		g_error_free (error);
 		g_free (filename_markup);
+		g_object_unref (builder);
 
 		return FALSE;
 	}
@@ -1330,7 +1371,7 @@ gedit_utils_drop_get_uris (GtkSelectionData *selection_data)
 	gint p = 0;
 	gchar **uri_list;
 
-	uris = g_uri_list_extract_uris ((gchar *) selection_data->data);
+	uris = g_uri_list_extract_uris ((gchar *) gtk_selection_data_get_data (selection_data));
 	uri_list = g_new0(gchar *, g_strv_length (uris) + 1);
 
 	for (i = 0; uris[i] != NULL; i++)
